#include <cmath>
#include <fstream>
#include "bmp24.h"
#include "misc.h"

using namespace std;

void loadBMP24(char *fileName, RGBColour *image)
{
   int x, y, count;
   ifstream inputFile(fileName, ios::in | ios::binary);
   BITMAPFILEHEADER *bmfh = new BITMAPFILEHEADER;
   BITMAPINFOHEADER *bmih = new BITMAPINFOHEADER;

   inputFile.read((char *) bmfh, BITMAPFILEHEADER::LENGTH);
   inputFile.read((char *) bmih, BITMAPINFOHEADER::LENGTH);
   if(bmih->biBitCount != 24) return; // Don't bother if it's not a 24bpp image.
   if(bmih->biCompression != 0) return; // Don't bother if it's compressed.
   char ch;
   for(y = (int) bmih->biHeight - 1; y >= 0; y--)
   {
      for(x = 0; x < (int) bmih->biWidth; x++)
      {
         inputFile.get(ch);
         image[x + y * bmih->biWidth].b = ((unsigned char) ch) / 255.0f;
         inputFile.get(ch);
         image[x + y * bmih->biWidth].g = ((unsigned char) ch) / 255.0f;
         inputFile.get(ch);
         image[x + y * bmih->biWidth].r = ((unsigned char) ch) / 255.0f;
      }
      if((bmih->biWidth & 3) != 0) for(count = 0; count < 4 - ((int) (3 * bmih->biWidth) & 3); count++) (void) inputFile.get();
   }
   inputFile.close();
   delete [] bmih;
   delete [] bmfh;
   return;
}

void saveBMP24(char *fileName, RGBColour *image, int width, int height)
{
   int bitmapHeaderLength = BITMAPFILEHEADER::LENGTH + BITMAPINFOHEADER::LENGTH;
   unsigned char bitmapHeader[] = { // Header for 640x480x24bpp uncompressed BMP file.
        0x42, 0x4D, 0x36, 0xB4, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x36, 0x04,
        0x00, 0x00, 0x28, 0x00, 0x00, 0x00, 0x80, 0x02, 0x00, 0x00, 0xE0, 0x01,
        0x00, 0x00, 0x01, 0x00, 0x18, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xB0,
        0x04, 0x00, 0xCE, 0x0E, 0x00, 0x00, 0xC4, 0x0E, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00 };

   sPutDW(bitmapHeader + 2, bitmapHeaderLength + 3 * width * height);
   sPutDW(bitmapHeader + 10, bitmapHeaderLength);
   sPutDW(bitmapHeader + 18, width);
   sPutDW(bitmapHeader + 22, height);
   sPutDW(bitmapHeader + 34, 3 * width * height);
   ofstream outputFile(fileName, ios::out | ios::binary);
   outputFile.write((char *) bitmapHeader, bitmapHeaderLength);

   int x, y;
   for(y = height - 1; y >= 0; y--) // I should really add padding because BMP files like to have row lengths that are divisible by 4.
      for(x = 0; x < width; x++)
      {
         outputFile.put(round(image[x + y * width].b * 255));
         outputFile.put(round(image[x + y * width].g * 255));
         outputFile.put(round(image[x + y * width].r * 255));
      }
   outputFile.close();
   return;
}
