#include "disc.h"
#include "misc.h"

Disc::Disc(float uRadius, float vRadius, Vector3D centre, Vector3D normal, Vector3D uAxis, int textureUCentre, int textureVCentre, int discUSize, int discVSize, RGBColour *texture, int textureWidth, int textureHeight, int specularExponent, float specularReflectivity)
:Primitive(texture, textureWidth, textureHeight, specularExponent, specularReflectivity)
{
   this->uRadius = uRadius;
   this->vRadius = vRadius;
   this->centre = centre;
   this->normal = normal;
   normal.normalise();
   this->uAxis = uAxis;
   uAxis.normalise();
   this->vAxis = normal.cross(uAxis);
   this->textureUCentre = textureUCentre;
   this->textureVCentre = textureVCentre;
   this->discUSize = discUSize;
   this->discVSize = discVSize;
}

float Disc::get_t(Ray ray)
{
   float t = ray.direction.dot(normal);
   if(t == 0.0) return nointersection_t_Value; // No intersection because the light ray is parallel to the plane in which the disc lies.
   t = (centre - ray.source).dot(normal) / t;
   Vector3D intersection = ray.source + ray.direction * t;
   float uCoord = (intersection - centre).dot(uAxis);
   float vCoord = (intersection - centre).dot(vAxis);
   if(uCoord * uCoord / (uRadius * uRadius) + vCoord * vCoord / (vRadius * vRadius) > 1) return nointersection_t_Value;
   return t;
}

RGBColour Disc::getColour(Vector3D intersection)
{
   float uCoord = intersection.dot(uAxis);
   float vCoord = intersection.dot(vAxis);
   int u = textureUCentre + round((uCoord + uRadius) / (2 * uRadius) * (discUSize - 1));
   int v = textureVCentre + round((vCoord + vRadius) / (2 * vRadius) * (discVSize - 1));
   return texture[u + v * textureWidth];
}

Vector3D Disc::getNormal(Vector3D intersection)
{
   return normal;
}
