#include "matrix3x3.h"
#include <cmath>

Matrix3x3::Matrix3x3(float *data)
{
   int i, j;
   for(i = 0; i < size; i++)
      for(j = 0; j < size; j++)
         this->data[i][j] = data[j + i * 3];
}

Matrix3x3::Matrix3x3(Vector3D row1, Vector3D row2, Vector3D row3)
{
   data[0][0] = row1.x;
   data[0][1] = row1.y;
   data[0][2] = row1.z;
   data[1][0] = row2.x;
   data[1][1] = row2.y;
   data[1][2] = row2.z;
   data[2][0] = row3.x;
   data[2][1] = row3.y;
   data[2][2] = row3.z;
}

Matrix3x3::Matrix3x3(float psi, float theta, float phi)
{  // NB: The matrix below is: zRot(psi) * yRot(theta) * zRot(phi)
   data[0][0] = cos(theta) * cos(phi) * cos(psi) - sin(phi) * sin(psi);
   data[0][1] = cos(theta) * sin(phi) * cos(psi) + cos(phi) * sin(psi);
   data[0][2] = - sin(theta) * cos(psi);
   data[1][0] = - cos(theta) * cos(phi) * sin(psi) - sin(phi) * cos(psi);
   data[1][1] = - cos(theta) * sin(phi) * sin(psi) + cos(phi) * cos(psi);
   data[1][2] = sin(theta) * sin(psi);
   data[2][0] = sin(theta) * cos(phi);
   data[2][1] = sin(theta) * sin(phi);
   data[2][2] = cos(theta);
}

Matrix3x3 Matrix3x3::operator*(Matrix3x3 argument)
{
   Matrix3x3 result;
   int i, j, k;
   for(i = 0; i < size; i++)
      for(j = 0; j < size; j++)
      {
         result.data[i][j] = 0;
         for(k = 0; k < size; k++)
            result.data[i][j] += data[i][k] * argument.data[k][j];
      }
   return result;
}

Vector3D Matrix3x3::operator*(Vector3D argument)
{
   Vector3D result(data[0][0] * argument.x + data[0][1] * argument.y + data[0][2] * argument.z,
                   data[1][0] * argument.x + data[1][1] * argument.y + data[1][2] * argument.z,
                   data[2][0] * argument.x + data[2][1] * argument.y + data[2][2] * argument.z);
   return result;
}

Matrix3x3 Matrix3x3::transpose()
{
   Matrix3x3 result;
   int i, j;
   for(i = 0; i < size; i++)
      for(j = 0; j < size; j++)
         result.data[i][j] = data[j][i];
   return result;
}
