#include "primitive.h"
#include <cmath>

Primitive::Primitive(RGBColour *texture, int textureWidth, int textureHeight, int specularExponent, float specularReflectivity)
{
   this->texture = texture;
   this->textureWidth = textureWidth;
   this->textureHeight = textureHeight;
   this->specularExponent = specularExponent;
   diffuseReflectivity = 1 - specularReflectivity;
   this->specularReflectivity = specularReflectivity;
}

RGBColour Primitive::getLocalLight(Vector3D intersection, Vector3D viewDirection, LightSource light)
{  // Phong illumination model. I must add light transmission and deal with the one versus two sided issue next.
   float temp;
   Vector3D normal = getNormal(intersection);
   RGBColour diffuseColour = getColour(intersection);
   RGBColour specularColour(1.0f, 1.0f, 1.0f); // This could have a texture too but this will do for now.

   // First, let's get the direct diffuse light.
   RGBColour directDiffuseLight;   
   temp = -normal.dot(light.direction);
   if(temp > 0) directDiffuseLight = diffuseColour * light.colour * temp;

   if(specularReflectivity == 0.0f) return directDiffuseLight;

   // Now, let's get the direct specular light.
   RGBColour directSpecularLight;
   viewDirection.normalise();

/*
The faster approximation to the Phong model is the Blinn approximation.
For this, instead of working with reflectedLight as below, we work with:
   Vector3D H = light.direction + viewDirection;
   H.normalise();
   temp = - normal.dot(H);
*/

/*
Normally people, calculate the specular light as m.v where m is the light vector reflected in the normal
and v is the view vector. However this is clearly the same as r.l where r is the view vector reflected
in the normal and I prefer this.
*/
   Vector3D reflectedView = viewDirection - normal * 2 * normal.dot(viewDirection);
   reflectedView.normalise();
   temp = - reflectedView.dot(light.direction);
   if(temp > 0)
   {
      temp = pow(temp, specularExponent);
      directSpecularLight = specularColour * light.colour * temp;
   }
   return directDiffuseLight * diffuseReflectivity + directSpecularLight * specularReflectivity;
}
