#include "rgbcolour.h"
#include "vector3d.h"
#include "matrix3x3.h"
#include "cylinder.h"
#include "disc.h"
#include "ellipsoid.h"
#include "triangle.h"
#include "primitive.h"
#include "lightsource.h"
#include "scene.h"
#include "bmp24.h"
#include "screen.h"
#include "camera.h"
#include "misc.h"
#include <cmath>

using namespace std;

void gammaCorrect(RGBColour *image, int size, float gamma);

int main()
{
   int textureWidth = 256, textureHeight = 256;
   RGBColour *stone = new RGBColour[textureHeight * textureWidth];
   RGBColour *marble = new RGBColour[textureHeight * textureWidth];
   RGBColour *wood = new RGBColour[textureHeight * textureWidth * 4];
   RGBColour *concrete = new RGBColour[textureHeight * textureWidth];
   RGBColour *copper = new RGBColour[textureHeight * textureWidth * 4];
   loadBMP24("textures/stone.bmp", stone);
   loadBMP24("textures/marble.bmp", marble);
   loadBMP24("textures/wood.bmp", wood);
   loadBMP24("textures/concrete.bmp", concrete);
   loadBMP24("textures/copper.bmp", copper);

   int radius = 50;
   Vector3D xAxis(1, 0, 0), yAxis(0, 0, 1), zAxis(0, 1, 0), centre;
   centre = Vector3D(3 * radius / 2, -radius * 2, -radius * 5 / 2);
   Cylinder testCylinder(radius / 4, radius * 4, centre, xAxis, yAxis, zAxis, concrete, textureWidth, textureHeight, 5, 0.4);

   Matrix3x3 transitionMatrix(PI / 3, PI / 4, PI / 4);
   centre = Vector3D(-3 * radius / 2, -radius, -radius);
   Ellipsoid testEllipsoid1(radius, radius * 1.5, radius, centre, transitionMatrix, stone, textureWidth, textureHeight, 35, 0.6);

   transitionMatrix = Matrix3x3(0, PI / 2, 0);
   centre = Vector3D(3 * radius / 2, 0, -radius);
   Ellipsoid testEllipsoid2(radius, radius, radius, centre, transitionMatrix, marble, textureWidth, textureHeight);

   centre = Vector3D(0, 0, 2 * radius);
   Ellipsoid testEllipsoid3(radius, radius, radius, centre, transitionMatrix, stone, textureWidth, textureHeight, 15, 0.6);

   centre = Vector3D(0, radius, 0);
   Vector3D normal(0, -1, 0);
   Vector3D uAxis(1, 0, 0);
   Disc testDisc(radius * 3, radius * 4, centre, normal, uAxis, 256, 256, 256, 256, copper, textureWidth * 2, textureHeight * 2, 1, 0.2);

   centre = Vector3D(0, 0, 0);
   Vector3D vertex1(3 * radius, -radius, radius);
   Vector3D vertex2(radius, -radius, radius);
   Vector3D vertex3(3 * radius, -radius, -radius);
   Vector3D vertex4(radius, -radius, -radius);
   Vector3D vertex5(2 * radius, -radius * 3, 0);
   
   Triangle face1(vertex1, vertex2, vertex3, 255, 255, 0, 255, 255, 0, wood, textureWidth, textureHeight);
   Triangle face2(vertex2, vertex4, vertex3, 0, 255, 0, 0, 255, 0, wood, textureWidth, textureHeight);
   Triangle face3(vertex1, vertex2, vertex5, 255, 255, 0, 255, 0, 0, wood, textureWidth, textureHeight);
   Triangle face4(vertex2, vertex4, vertex5, 0, 255, 255, 0, 255, 255, wood, textureWidth, textureHeight);
   Triangle face5(vertex4, vertex3, vertex5, 0, 0, 255, 0, 0, 255, wood, textureWidth, textureHeight);
   Triangle face6(vertex3, vertex1, vertex5, 255, 0, 255, 255, 0, 0, wood, textureWidth, textureHeight);

//   LightSource testLight1(Vector3D(0, 0.5, 0.8660254), RGBColour(0.82, 0.79, 0.18));
//   LightSource testLight2(Vector3D(0.7071067, 0.7071067, 0), RGBColour(0.0, 0.0, 0.8));
   LightSource testLight1(Vector3D(0, 0.5, 0.8660254), RGBColour(0.19, 0.5, 0.38));
   LightSource testLight2(Vector3D(0.7071067, 0.7071067, 0), RGBColour(0.1, 0.0, 0.7));
   LightSource testLight3(Vector3D(0, 0.5, -0.8660254), RGBColour(0.75, 0.4, 0.0));

   Scene testScene(RGBColour(0.2, 0.2, 0.3));
   testScene.addLightSource(&testLight1);
   testScene.addLightSource(&testLight2);
   testScene.addLightSource(&testLight3);

   testScene.addPrimitive(&testCylinder);
   testScene.addPrimitive(&testEllipsoid1);
   testScene.addPrimitive(&testEllipsoid2);
   testScene.addPrimitive(&testEllipsoid3);
   testScene.addPrimitive(&testDisc);
   testScene.addPrimitive(&face1);
   testScene.addPrimitive(&face2);
   testScene.addPrimitive(&face3);
   testScene.addPrimitive(&face4);
   testScene.addPrimitive(&face5);
   testScene.addPrimitive(&face6);

//   Vector3D origin(0, -radius * 3, radius * 5);
//   float angle = -7 * PI / 6;
   Vector3D origin(0, -radius * 3, -radius * 5);
   float angle = PI / 6;
   xAxis = Vector3D(1, 0, 0);
   yAxis = Vector3D(0, cos(angle), -sin(angle));
   zAxis = Vector3D(0, sin(angle), cos(angle));
   Matrix3x3 cameraMatrix(xAxis, yAxis, zAxis);
   Camera3D testCamera(origin, cameraMatrix);

   RGBColour background(0, 0, 0);
   Screen testScreen(1024, 768, background);

   Ray ray(testCamera.position, Vector3D(0, 0, 0));
   int x, y;
   Primitive *Obj;
   float t;

   for(x = - testScreen.screenWidth / 2; x < testScreen.screenWidth / 2; x++)
      for(y = - testScreen.screenHeight / 2; y < testScreen.screenHeight / 2; y++)
      {
         ray.direction = Vector3D(testCamera.fromCameraSpace(Vector3D(x, y, testScreen.screenCameraDistance)) - ray.source);
         if((t = testScene.get_t(ray, Obj)) > 0)
            testScreen.putPixel(x + testScreen.screenWidth / 2, y + testScreen.screenHeight / 2, testScene.doLighting(ray.source + ray.direction * t, ray.direction, *Obj));
      }
//   gammaCorrect(testScreen.image, testScreen.screenWidth * testScreen.screenHeight, 1.7);
   saveBMP24("image.bmp", testScreen.image, testScreen.screenWidth, testScreen.screenHeight);
   delete [] stone;
   delete [] marble;
   delete [] wood;
   delete [] concrete;
   delete [] copper;
   return 0;
}

void gammaCorrect(RGBColour *image, int size, float gamma)
{
   RGBColour p;
   int i;

   for(i = 0; i < size; i++)
   {
      p.r = pow(image[i].r, gamma);
      p.g = pow(image[i].g, gamma);
      p.b = pow(image[i].b, gamma);
      image[i] = p;
   }
   return;
}
