#include "triangle.h"
#include "misc.h"

Triangle::Triangle(Vector3D vertex1, Vector3D vertex2, Vector3D vertex3, int textureIndexU1, int textureIndexV1, int textureIndexU2, int textureIndexV2, int textureIndexU3, int textureIndexV3, RGBColour *texture, int textureWidth, int textureHeight, int specularExponent, float specularReflectivity)
:Primitive(texture, textureWidth, textureHeight, specularExponent, specularReflectivity)
{
   this->vertex1 = vertex1;
   this->vertex2 = vertex2;
   this->vertex3 = vertex3;
   this->textureIndexU1 = textureIndexU1;
   this->textureIndexV1 = textureIndexV1;
   this->textureIndexU2 = textureIndexU2;
   this->textureIndexV2 = textureIndexV2;
   this->textureIndexU3 = textureIndexU3;
   this->textureIndexV3 = textureIndexV3;
   normal = (vertex1 - vertex2).cross(vertex1 - vertex3);
   normal.normalise();
   sign123 = lineSide(vertex1, vertex2, vertex3);
   sign231 = lineSide(vertex2, vertex3, vertex1);
   sign312 = lineSide(vertex3, vertex1, vertex2);
}

int Triangle::inside(Vector3D point)
{
   int sign;
   sign = lineSide(point, vertex2, vertex3);
   if(sign != sign123) return 0;
   sign = lineSide(point, vertex3, vertex1);
   if(sign != sign231) return 0;
   sign = lineSide(point, vertex1, vertex2);
   if(sign != sign312) return 0;
   return 1;
}

int Triangle::lineSide(Vector3D point, Vector3D linePoint1, Vector3D linePoint2)
{
   return (point - linePoint1).dot((linePoint1 - linePoint2).cross(normal)) >= 0.0;
}

float Triangle::get_t(Ray ray)
{
   float t = ray.direction.dot(normal);
   if(t == 0.0) return nointersection_t_Value; // No intersection because the light ray is parallel to the plane.
   t = (vertex1 -  ray.source).dot(normal) / t; // I could use vertex2 or vertex3 instead of vertex1 here of course since the equation of a plane is r.n = const
   Vector3D intersection = ray.source + ray.direction * t;
   if(!inside(intersection) || t < 0) return nointersection_t_Value; // No intersection because the point of intersection isn't both inside the triangle boundary and in front of the source.
   return t;
}

RGBColour Triangle::getColour(Vector3D intersection)
{
   Vector3D v12 = vertex2 - vertex1, v13 = vertex3 - vertex1, p1 = intersection - vertex1;
   float t12, t13;
   float t = v12.cross(v13).length();
   t *= t;
   t12 = (v13.dot(v13) * p1.dot(v12) - v12.dot(v13) * p1.dot(v13)) / t;
   t13 = (- v12.dot(v13) * p1.dot(v12) + v12.dot(v12) * p1.dot(v13)) / t;
   int u = round(textureIndexU1 +
                (textureIndexU2 - textureIndexU1) * t12 +
                (textureIndexU3 - textureIndexU1) * t13);
   int v = round(textureIndexV1 +
                (textureIndexV2 - textureIndexV1) * t12 +
                (textureIndexV3 - textureIndexV1) * t13);
   return texture[u + v * textureWidth];
}

Vector3D Triangle::getNormal(Vector3D intersection)
{
   return normal;
}
